#' SEIQHRF function
#'
#' Function to implement infection processes for SEIQHRF model
#'
#' @param init Initial state values, returned by \code{\link{init_seiqhrf}}.
#' @param control Control parameters of the model, returned by \code{\link{control_seiqhrf}}.
#' @param param Model parameters, returned by \code{\link{param_seiqhrf}}. 
#' @param priors A prior object returned by \code{\link{select_prior}}.
#' @param seed random seed for checking consistency
#'
#' @return Updated dat
#' 
#' @importFrom stats rnorm
#' @importFrom stats rbeta
#' @importFrom foreach "%dopar%" foreach
#' @importFrom EpiModel verbose.icm
#' @importFrom future availableCores
#' @export
seiqhrf <- function(init, control, param, priors = NULL, seed = NULL) {
  
  crosscheck_seiqhrf(param, init, control, priors)
  nsims <- control$nsims
  ncores <- ifelse(control$nsims == 1, 1, min(future::availableCores(), control$ncores))
  control$ncores <- ncores
  use_prior <- FALSE
  param_inuse <- param
  nsteps <- control$nsteps 
  
  if(!is.null(priors)){
    
    use_prior <- TRUE
    sampled_pars <- list()
    
    par_names <- names(priors)[names(priors) != "shift_only"]
    for(par in par_names){
      sampled_pars[[par]] <- simulate_pars(priors[[par]], param[[par]], nsims, nsteps, priors$shift_only)
    }
    
  }
  
  doParallel::registerDoParallel(ncores)
  
  sout <- foreach(s = 1:nsims) %dopar% {
    
    for(par in names(priors)){
      param_inuse[[par]] <- sampled_pars[[par]][s, ]
    }
    
    control$nsims <- 1
    control$currsim <- s
    
    ## Initialization module
    if (!is.null(control[["initialize.FUN"]])) {
      dat <- do.call(control[["initialize.FUN"]], list(param_inuse, init, control, seed))
    }
    
    # Timestep loop
    for (at in 2:nsteps) {
      
      ## Infection: S -> E
      if (!is.null(control[["infection.FUN"]])) {
        dat <- do.call(control[["infection.FUN"]], list(dat, at, seed))
      }
      
      
      ## Recovery
      if (!is.null(control[["recovery.FUN"]])) {
        dat <- do.call(control[["recovery.FUN"]], list(dat, at, seed))
      }
      
      
      ## Departure Module
      if (!is.null(control[["departures.FUN"]])) {
        dat <- do.call(control[["departures.FUN"]], list(dat, at, seed))
      }
      
      
      ## Arrival Module
      if (!is.null(control[["arrivals.FUN"]])) {
        dat <- do.call(control[["arrivals.FUN"]], list(dat, at, seed))
      }
      
      
      ## Outputs
      if (!is.null(control[["get_prev.FUN"]])) {
        dat <- do.call(control[["get_prev.FUN"]], list(dat, at))
      }
      
      
      ## Track progress
      EpiModel::verbose.icm(dat, type = "progress", s, at)
    }
    
    dat$param <- param
    # Set output
    out <- saveout_seiqhrf(dat, s=1)
    return(out)
    
  }
  
  # aggregate results collected from each thread
  collected_times <- list()
  
  # collect the times from sout then delete them
  for (i in 1:length(sout)) {
    collected_times[[paste0("sim", i)]] <- sout[[i]]$times$sim1
    sout[[i]]$times <- NULL
  }
  
  # merge $epi structures
  merged.out <- sout[[1]]
  if(length(sout) > 1){
    for (i in 2:length(sout)) {
      merged.out <- merge.seiqhrf.icm(merged.out, sout[[i]], param.error = FALSE)
    } 
  }
  out <- merged.out
  
  # add the collected timing data
  out$times <- collected_times
  
  if(use_prior)  out$sampled.pars <- sampled_pars

  
  class(out) <- "seiqhrf"
  
  return(out)
}
