#' Module to simulate "departure"
#'
#' Function to handle background demographics for the SEIQHRF model.
#' Specifically departures (deaths not due to the virus, and emigration).
#'
#' @param dat Object containing all data
#' @param at Step number
#' @param seed random seed for checking consistency
#'
#' @return Updated \code{dat}.
#' @keywords internal

departures.FUN <- function(dat, at, seed = NULL) {
    
    if(!is.null(seed)) set.seed(seed)
    
    # Conditions --------------------------------------------------------------
    if (!dat$param$vital) return(dat)
    
    # Variables -----------------------------------------------------------------
    rate <- dat$param$ds.rate
    rand <- dat$control$d.rand
    status <- dat$attr$status
    active <- dat$attr$active
    
    # Susceptible departures ------------------------------------------------------
    res <- update_active(rate, rand, active, status, label = "s")
    nDepartures <- res[[1]]
    if(!is.null(res[[2]]))  active <- dat$attr$active[res[[2]]] <- 0
    
    if(dat$control$track.departure){
        if (at == 2) dat$epi$sd.flow <- c(0, nDepartures) else dat$epi$sd.flow[at] <- nDepartures
    }
    
    # Exposed Departures ---------------------------------------------------------
    res <- update_active(rate, rand, active, status, label = "e")
    nDepartures <- res[[1]]
    if(!is.null(res[[2]]))  active <- dat$attr$active[res[[2]]] <- 0
    
    if(dat$control$track.departure){
        if (at == 2) dat$epi$ed.flow <- c(0, nDepartures) else dat$epi$ed.flow[at] <- nDepartures
    }
    
    # Infected Departures ---------------------------------------------------------
    res <- update_active(rate, rand, active, status, label = "i")
    nDepartures <- res[[1]]
    if(!is.null(res[[2]]))  active <- dat$attr$active[res[[2]]] <- 0
    
    if(dat$control$track.departure){
        if (at == 2) dat$epi$id.flow <- c(0, nDepartures) else dat$epi$id.flow[at] <- nDepartures 
    }
    
    # Quarantined Departures ---------------------------------------------------------
    res <- update_active(rate, rand, active, status, label = "q")
    nDepartures <- res[[1]]
    if(!is.null(res[[2]]))  active <- dat$attr$active[res[[2]]] <- 0
    
    if(dat$control$track.departure){
        if (at == 2) dat$epi$qd.flow <- c(0, nDepartures) else dat$epi$qd.flow[at] <- nDepartures
    }
    
    # Hospitalised Departures ---------------------------------------------------------
    res <- update_active(rate, rand, active, status, label = "h")
    nDepartures <- res[[1]]
    if(!is.null(res[[2]]))  active <- dat$attr$active[res[[2]]] <- 0
    
    if(dat$control$track.departure){
        if (at == 2) dat$epi$hd.flow <- c(0, nDepartures) else dat$epi$hd.flow[at] <- nDepartures
    }
    
    # Recovered Departures --------------------------------------------------------
    res <- update_active(rate, rand, active, status, label = "r")
    nDepartures <- res[[1]]
    if(!is.null(res[[2]]))  active <- dat$attr$active[res[[2]]] <- 0
    
    if(dat$control$track.departure){
        if (at == 2) dat$epi$rd.flow <- c(0, nDepartures) else dat$epi$rd.flow[at] <- nDepartures
    }
    
    # return --------------------------------------------------------
    return(dat)
}
