#' Module to simulate "progress" from compartment E to I
#'
#' @param dat Object containing all data
#' @param at time point
#' @param seed random seed for checking consistency
#'
#' @return updated \code{dat}
#' @importFrom stats pweibull
#' @importFrom stats rbinom
#' @importFrom EpiModel ssample
#' @importFrom stats rgeom
#' @importFrom stats sd
#' @keywords internal

recovery.FUN <- function(dat, at, seed = NULL) {
  
  #cat("at: ", at, "\n")
  
  if(!is.null(seed)) set.seed(seed)
  # Conditions --------------------------------------------------------------------------
  if (dat$control$type != "SEIQHRF")  return(dat)
  
  # Variables ---------------------------------------------------------------------------
  active <- dat$attr$active
  status <- dat$attr$status
  groups <- dat$param$groups
  group <- dat$attr$group
  type <- dat$control$type
  expTime <- dat$attr$expTime
  
  # ------------------------- progress from exposed to infectious (E -> I)--------------------------
  res <- update_status(rate = dat$param$prog.rate,
                       rand = dat$control$prog.rand, 
                       active, dat$attr$status, 
                       label = "e", 
                       state =  "i", 
                       at, expTime, 
                       prog.dist.scale = dat$param$prog.dist.scale,
                       prog.dist.shape = dat$param$prog.dist.shape)
  
  nProg <- res[[1]]
  if(!is.null(res[[2]])) dat$attr$infTime[res[[2]]] <- at
  status <- res[[3]]
  dat$attr$status <- status
  
  #cat("finished prog \n")
  
  if (type %in% c("SEIQHR", "SEIQHRF")) {
    
    # ----- quarantine (I -> Q)------- 
    quar.rate <- dat$param$quar.rate
    rate <- ifelse(length(quar.rate) > 1, quar.rate[at], quar.rate)
    
    res <- update_status(rate,
                         rand = dat$control$quar.rand,
                         active = dat$attr$active, 
                         status = dat$attr$status,
                         label = "i", 
                         state = "q",
                         at, expTime, 
                         prog.dist.scale = dat$param$quar.dist.scale,
                         prog.dist.shape = dat$param$quar.dist.shape)
    nQuar <- res[[1]]
    if(!is.null(res[[2]])) dat$attr$quarTime[res[[2]]] <- at
    status <- res[[3]]
    dat$attr$status <- status
    
    # ----- need to be hospitalised (I, Q -> H)-------
    rate <- rep(dat$param$hosp.rate, 
                sum(dat$attr$active == 1 & (dat$attr$status == "i" | dat$attr$status == "q")))
    
    res <- update_status(rate,
                         rand = dat$control$hosp.rand,
                         active = dat$attr$active,
                         status = dat$attr$status, 
                         label = c("i", "q"), 
                         state = "h", 
                         at, expTime, 
                         prog.dist.scale = dat$param$hosp.dist.scale,
                         prog.dist.shape = dat$param$hosp.dist.shape)
    
    nHosp <- res[[1]]
    dat$attr$hospTime[res[[2]]] <- at
    status <- res[[3]]
    dat$attr$status <- status
    
    # ------------------------- discharge from need to be hospitalised (H -> R)---------------------------
    recovState <- "r"
    disch.rate <- dat$param$disch.rate
    dcrate <- ifelse(length(disch.rate) > 1, disch.rate[at], disch.rate)
    
    res <- update_status(rate = dcrate,
                         rand = dat$control$disch.rand,
                         active = dat$attr$active, 
                         status = dat$attr$status,
                         label = "h", 
                         state = recovState, 
                         at, expTime, 
                         prog.dist.scale = dat$param$disch.dist.scale,
                         prog.dist.shape = dat$param$disch.dist.shape)
    
    nDisch <- res[[1]]
    dat$attr$dischTime[res[[2]]] <- at
    status <- res[[3]]
    dat$attr$status <- status
    #cat("finished discharge \n")
  }
  
  # --------------------------------------- recover (I, Q -> R)-------------------------------------------------
  res <- update_status(rate = dat$param$rec.rate,
                       rand = dat$control$rec.rand, 
                       active, 
                       status = dat$attr$status, 
                       label = c("i", "q"), 
                       state = recovState, 
                       at, 
                       expTime = dat$attr$expTime, 
                       prog.dist.scale = dat$param$rec.dist.scale,
                       prog.dist.shape = dat$param$rec.dist.shape)
  
  nRecov <- res[[1]]
  dat$attr$recovTime[res[[2]]] <- at
  status <- res[[3]]
  dat$attr$status <- status
  
  # --------------------------------------- recover (E-> R)-------------------------------------------------
  res <- update_status(rate = dat$param$arec.rate,
                       rand = dat$control$arec.rand, 
                       active, 
                       status = dat$attr$status, 
                       label = c("e"), 
                       state = recovState, 
                       at, 
                       expTime = dat$attr$expTime, 
                       prog.dist.scale = dat$param$arec.dist.scale,
                       prog.dist.shape = dat$param$arec.dist.shape)
  
  nRecov2 <- res[[1]]
  dat$attr$recovTime[res[[2]]] <- at
  status <- res[[3]]
  dat$attr$status <- status
  
  if (type %in% c("SEIQHRF")) {
    # -------------------------------------- case fatality (H -> F)----------------------------------------
    ## obtain rates
    idsElig <- which(active == 1 & status =="h")
    if(length(idsElig) > 0){
      hosp.cap <- dat$param$hosp.cap
      rates <- dat$param$fat.rate.base
      gElig <- group[idsElig]
      timeInHospElig <- at - dat$attr$hospTime[idsElig]
      h.num.yesterday <- 0
      if (!is.null(dat$epi$h.num[at - 1])) {
        h.num.yesterday <- dat$epi$h.num[at - 1]
        if (h.num.yesterday > hosp.cap) {
          blended.rate <- ((hosp.cap * rates) +
                             ((h.num.yesterday - hosp.cap) * dat$param$fat.rate.overcap)) /
            h.num.yesterday
          rates <- blended.rate
        }
      }
      ratesElig <- rates[gElig]
      ratesElig <- ratesElig + timeInHospElig*dat$param$fat.tcoeff*ratesElig
    }
    
    res <- update_status(rate = ratesElig,
                         rand = dat$control$fat.rand,
                         active = dat$attr$active, 
                         status = dat$attr$status,
                         label = "h", 
                         state = "f", at)
    nFat <- res[[1]]
    if(!is.null(res[[2]])) dat$attr$fatTime[res[[2]]] <- at
    status <- res[[3]]
    dat$attr$status <- status
    
    #cat("finished fatality \n")
  }
  
  # Output Summary statistics------------------------------------------------------------------
  if(dat$control$track.flow){
    if (at == 2) {
      dat$epi[["iq2r.flow"[1]]] <- c(0, nRecov)
      dat$epi[["er.flow"[1]]] <- c(0, nRecov2)
      dat$epi[["ei.flow"[1]]] <- c(0, nProg)
      dat$epi[["iq.flow"[1]]] <- c(0, nQuar)
      dat$epi[["iq2h.flow"[1]]] <- c(0, nHosp)
      dat$epi[["hf.flow"[1]]] <- c(0, nFat)
      dat$epi[["hr.flow"[1]]] <- c(0, nDisch)
    } else {
      dat$epi[["iq2r.flow"[1]]][at] <- nRecov
      dat$epi[["er.flow"[1]]][at] <- nRecov2
      dat$epi[["ei.flow"[1]]][at] <- nProg
      dat$epi[["iq.flow"[1]]][at] <- nQuar
      dat$epi[["iq2h.flow"[1]]][at] <- nHosp
      dat$epi[["hf.flow"[1]]][at] <- nFat
      dat$epi[["hr.flow"[1]]][at] <- nDisch
    }
  }
  
  return(dat)
}
