#' Control Settings for Stochastic Individual Contact Models
#'
#' Sets the controls for stochastic individual contact models simulated 
#' with \code{\link{seiqhrf}}. Similar to EpiModel::control.icm, but allows for
#' model types with additional compartments.
#' 
#'
#' @param nsteps The number of time points to be simulated. 
#' @param nsims The number of simulations to run.
#' @param prog.rand Method for progression from E compartment to I. If TRUE, 
#'         random binomial draws at \code{prog.rate}, if FALSE, random draws from a 
#'         Weibull distribution, with parameters \code{prog.dist.scale} and \code{prog.dist.shape}.
#' @param rec.rand Method for progression from I compartment to R.
#'       If \code{TRUE}, use a stochastic recovery model, with the
#'        number of recovered at each time step a random draw from
#'        Binomial distribution with success probability \code{rec.rate}.
#'        If FALSE, random draws from a 
#'         Weibull distribution, with parameters \code{rec.dist.scale} and \code{rec.dist.shape}.
#' @param quar.rand Method for self-isolation transition from I to Q. If TRUE, 
#'          random binomial draws at \code{quar.rate}, if FALSE, random draws from a 
#'         Weibull distribution, with parameters \code{quar.dist.scale} and \code{quar.dist.shape}.      .
#' @param hosp.rand Method for transition from I or Q to H -- that is, from 
#'          infectious or from self-isolated to requiring hospitalisation. If 
#'          TRUE, random binomial draws at \code{hosp.rate}, if FALSE, random draws from a 
#'         Weibull distribution, with parameters \code{hosp.dist.scale} and \code{hosp.dist.shape}.
#' @param disch.rand Method for transition from H to R -- that is, from 
#'          requiring hospitalisation to recovered. If TRUE, random binomial 
#'          draws at \code{disch.rate}, if FALSE, random draws from a 
#'         Weibull distribution, with parameters \code{disch.dist.scale} and \code{disch.dist.shape}.
#'          Note that the only way out of the H 
#'          compartment is recovery or death.
#' @param fat.rand Method for case fatality transition from H to F. If TRUE, 
#'          random binomial draws at fat.rate.base, if FALSE, random sample with
#'          a sample fraction also given by \code{fat.rate.base}. However, if the 
#'          current number of patients in the H (needs hospitalisation) 
#'          compartment is above a hospital capacity level specified by 
#'          hosp.cap, then the fatality rate is the mean of the base fatality 
#'          rate weighted by the hospital capacity, plus a higher rate, 
#'          specified by \code{fat.rate.overcap}, weighted by the balance of those 
#'          requiring hospitalisation (but who can't be accommodated). By 
#'          setting \code{fat.rate.overcap} higher, the effect of exceeding the 
#'          capacity of the health care system can be simulated. There is also
#'          a coefficient \code{fat.tcoeff} for the fatality rates that increases them 
#'          as a linear function of the number of days the individual has been 
#'          in the H compartment. Use of the co-efficient better approximates 
#'          the trapezoid survival time distribution typical of ICU patients.
#' @param arec.rand  Method for recovery transition from E to R. If TRUE, 
#'         random binomial draws at \code{arec.rate}, if FALSE, random draws from a 
#'         random draws from a Weibull distribution, with parameters 
#'         \code{arec.dist.scale} and \code{arec.dist.shape}.
#' @param a.rand If \code{TRUE}, use a stochastic arrival model, with the
#'        number of arrivals at each time step a function of random draws from a
#'        binomial distribution with the probability equal to the governing arrival
#'        rates. If \code{FALSE}, a deterministic rounded count of the
#'        expectation implied by those rates.
#' @param d.rand If \code{TRUE}, use a stochastic departure model, with the number of
#'        departures at each time step a function of random draws from a binomial
#'        distribution with the probability equal to the governing departure rates.
#'        If \code{FALSE}, a deterministic rounded count of the expectation
#'        implied by those rates.
#' @param verbose If \code{TRUE}, print model progress to the console.
#' @param verbose.int Time step interval for printing progress to console, where
#'        0 (the default) prints completion status of entire simulation and
#'        positive integer \code{x} prints progress after each \code{x} time
#'        steps.
#' @param skip.check If \code{TRUE}, skips the default error checking for the
#'        structure and consistency of the parameter values, initial conditions,
#'        and control settings before running base epidemic models. Setting
#'        this to \code{FALSE} is recommended when running models with new modules
#'        specified.
#' @param ncores Number of physical CPU cores used for parallel computation.
#' @param track.flows If TRUE, function \code{\link{seiqhrf}} returns the number of changes in each procedure transition (See model description in vignette). 
#' @param track.arrival If TRUE, function \code{\link{seiqhrf}} returns the number of departures of all compartments 
#' in every time step in each simulation.
#' @param track.departure  Similar to track.departure. 
#'
#' @section New Modules:
#' Base ICM models use a set of module functions that specify
#' how the individual agents in the population are subjected to infection, recovery,
#' demographics, and other processes. Core modules are those listed in the
#' \code{.FUN} arguments. For each module, there is a default function used in
#' the simulation. The default infection module, for example, is contained in
#' the \code{\link{infection.FUN}} function.
#' For original models, one may substitute replacement module functions for any of
#' the default functions. 
#'
#' @keywords control
#'
#' @return A \code{control.seiqhrf} object containing required control parameters and module functions.
#' @export
control_seiqhrf <- function(nsteps = 366, nsims = 8, 
                            prog.rand = FALSE, quar.rand = TRUE, hosp.rand = TRUE,
                            disch.rand = TRUE, rec.rand = FALSE, arec.rand = TRUE,
                            fat.rand = TRUE, a.rand = TRUE, d.rand = TRUE, 
                            verbose = FALSE, verbose.int = 0, skip.check = FALSE, 
                            ncores = 4, track.flows = TRUE, track.departure = FALSE, track.arrival = FALSE) {

  # Get arguments
  p <- list()
  passed <- names(as.list(match.call())[-1])
  p$usr.specified <- passed
  
  formal.args <- formals(sys.function())
  for (arg in names(formal.args)) {
    if (as.logical(mget(arg) != "")) {
      p[arg] <- list(get(arg))
    }
  }

 p$initialize.FUN <- "initialize.FUN"
 p$infection.FUN <- "infection.FUN"
 p$recovery.FUN <- "recovery.FUN"
 p$departures.FUN <- "departures.FUN"
 p$arrivals.FUN <- "arrivals.FUN"
 p$get_prev.FUN <- "get_prev.FUN"
 
  ## Defaults and checks
  p$type <- "SEIQHRF"
  if (is.null(p$nsteps)) stop("Specify nsteps", call. = FALSE)
  if (is.null(p$nsims)) stop("Specify nsims", call. = FALSE)

  ## Output
  class(p) <- c("control.seiqhrf", "list")
  return(p)
}
