#' @title Epidemic Parameters for Stochastic Individual Contact Models
#'
#' @description Sets the epidemic parameters for stochastic individual contact
#'              models simulated with \code{seiqhrf}.
#'              All parameters can either be a scalar or a vector of the length of the number of time steps to be simulated. 
#' @param inf.prob.e Probability of passing on infection at each
#'     exposure event for interactions between infectious people in the E
#'     compartment and susceptible in S. Note the default is lower than for
#'     inf.prob.i reflecting the reduced infectivity of infected but
#'     asymptomatic people (the E compartment). Otherwise as for \code{inf.exp.i}.
#' @param act.rate.e The number of exposure events (acts) between
#'     infectious individuals in the E compartment and susceptible individuals
#'     in the S compartment per day.
#' @param  inf.prob.i Probability of passing on infection at each
#'     exposure event for interactions between infectious people in the I
#'     compartment and susceptible in S. Reducing \code{inf.prob.i} is equivalent to
#'     increasing hygiene measures, such as not putting hands in eyes, nose or
#'     moth, use of hand sanitisers, wearing masks by the infected, and so on.
#' @param act.rate.i The average number of exposure events (acts) between
#'     infectious individuals in the I compartment and susceptible individuals
#'     in the S compartment per day.  Note that not every
#'     exposure event results in infection - that is governed by the \code{inf.prob.i}
#'     parameters (see below). Reducing \code{act.rate.i} is equivalent to increasing
#'     social distancing by people in the I compartment.
#' @param inf.prob.q  Probability of passing on infection at each
#'     exposure event for interactions between infectious people in the Q
#'     compartment and susceptible in S. Note the default is lower than for
#'     \code{inf.prob.i} reflecting the greater care that self-isolated individuals
#'     will, on average, take regarding hygiene measures, such as wearing masks,
#'     to limit spread to others.
#' @param act.rate.q The number of exposure events (acts) between
#'     infectious individuals in the Q compartment (isolated, self or otherwise)
#'     and susceptible individuals in the S compartment, per day. Note the much
#'     lower rate than for the I and E compartments, reflecting the much
#'     greater degree of social isolation for someone in (self-)isolation. The
#'     exposure event rate is not zero for this group, just much less.
#' @param quar.rate Rate per day at which symptomatic (or tested
#'     positive), infected I compartment people enter self-isolation (Q
#'     compartment). Asymptomatic E compartment people can't enter
#'     self-isolation because they don't yet know they are infected. Default is
#'     a low rate reflecting low community awareness or compliance with
#'     self-isolation requirements or practices, but this can be tweaked when
#'     exploring scenarios.
#' @param quar.dist.scale Scale parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{quar.rand == FALSE}.
#' @param quar.dist.shape Shape parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{quar.rand == FALSE}. 
#' @param hosp.rate Rate per day at which symptomatic (or tested
#'     positive), infected I compartment people or self-isolated Q compartment
#'     people enter the state of requiring hospital care -- that is, become
#'     serious cases. A default rate of 1% per day with an average illness
#'     duration of about 10 days means a bit less than 10% of cases will
#'     require hospitalisation.
#'     Only useful if parameter \code{quar.rand == TRUE}. 
#' @param hosp.dist.scale Scale parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{hosp.rand == FALSE}.
#' @param hosp.dist.shape Shape parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{hosp.rand == FALSE}. 
#' @param disch.rate Rate per day at which people needing
#'     hospitalisation recover, only useful if parameter \code{disch.rand == TRUE}.
#' @param disch.dist.scale Scale parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{disch.rand == FALSE}.
#' @param disch.dist.shape Shape parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{disch.rand == FALSE}. 
#' @param prog.rate Rate per day at which people who are infected
#'     but asymptomatic (E compartment) progress to becoming symptomatic (or
#'     test-positive), the I compartment.
#'     Only useful if parameter \code{prog.rand == TRUE}.
#' @param prog.dist.scale Scale parameter for Weibull distribution
#'     for progression, only useful if parameter \code{prog.rand == FALSE}.
#' @param prog.dist.shape Shape parameter for Weibull distribution
#'     for progression, only useful if parameter \code{prog.rand == FALSE}.
#' @param rec.rate Rate per day at which people who are infected and
#'     symptomatic (I compartment) recover, thus entering the R compartment.
#'     Only useful if parameter \code{rec.rand == TRUE}.
#' @param rec.dist.scale Scale parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{rec.rand == FALSE}.
#' @param rec.dist.shape Shape parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{rec.rand == FALSE}.
#' @param arec.rate Rate per day at which people who are exposed but asymptotic 
#'     (E compartment) recover, thus entering the R compartment.
#'      Only useful if parameter \code{arec.rand == TRUE}.
#' @param arec.dist.scale Scale parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{arec.rand == FALSE}.
#' @param arec.dist.shape Shape parameter for Weibull distribution for
#'     recovery, only useful if parameter \code{arec.rand == FALSE}.
#' @param fat.rate.base Baseline mortality rate per day for people
#'     needing hospitalisation (deaths due to the virus). 
#'     Only useful if parameter \code{fat.rand == TRUE}.
#' @param hosp.cap Number of available hospital beds for the modelled
#'     population. See parameter \code{fat.rand} for more details.
#' @param fat.rate.overcap Mortality rate per day for people needing
#'     hospitalisation but who can't get into hospital due to the hospitals
#'     being full (see parameters \code{hosp.cap} and \code{fat.rand}). The default rate is twice that
#'     for those who do get into hospital.
#' @param fat.tcoeff Time co-efficient for increasing mortality rate
#'     as time in the H compartment increases for each individual in it. 
#'     See parameter \code{fat.rand} for details.
#' @param vital Enables demographics, that is, arrivals and
#'     departures, to and from the simulated population.
#' @param flare.inf.point (Either a vector or a scalar) Time points where there is a sudden arrival of I's. 
#' @param flare.inf.num (same dimension as flare.inf.point) The number of I's arriving at the specified time points in flare.inf.point.
#' @param a.rate Background demographic arrival rate. Currently all
#'     arrivals go into the S compartment, the default is approximately the
#'     daily birth rate for Australia. Will be extended to cover immigration in
#'     future versions.
#' @param a.prop.e Arrivals proportion that goes to E (immigration). 
#' @param a.prop.i Arrivals proportion that goes to I (immigration). 
#' @param a.prop.q Arrivals proportion that goes to Q (immigration). 
#' @param ds.rate Background demographic departure (death not due to
#'     virus) rates. Defaults based on Australian crude death rates. Can be
#'     used to model emigration as well as deaths.
#' @param de.rate Background demographic departure (death not due to
#'     virus) rates. Defaults based on Australian crude death rates. Can be
#'     used to model emigration as well as deaths.
#' @param di.rate Background demographic departure (death not due to
#'     virus) rates. Defaults based on Australian crude death rates. Can be used
#'     to model emigration as well as deaths.
#' @param dq.rate Background demographic departure (death not due to
#'     virus) rates. Defaults based on Australian crude death rates. Can be used
#'     to model emigration as well as deaths.
#' @param dh.rate Background demographic departure (death not due to
#'     virus) rates. Defaults based on Australian crude death rates. Can be used
#'     to model emigration as well as deaths.
#' @param dr.rate Background demographic departure (death not due to
#'     virus) rates. Defaults based on Australian crude death rates. Can be used
#'     to model emigration as well as deaths.
#'
#' @return A \code{param.seiqhrf} object with 
#'  epidemic parameters for the stochastic individual
#' contact models simulated with the \code{\link{seiqhrf}} function. 
#' The model specification will be chosen as a result of
#' the model parameters entered here and the control settings in
#' \code{\link{control_seiqhrf}}.
#' 
#' @seealso Use \code{\link{init_seiqhrf}} to specify the initial conditions and
#'          \code{\link{control_seiqhrf}} to specify the control settings. Run the
#'          parameterized model with \code{\link{seiqhrf}}.
#'
#' @keywords parameterization
#'
#' @export
#'
param_seiqhrf <- function(inf.prob.e = 0.02,
                          act.rate.e = 10,
                          inf.prob.i = 0.05,
                          act.rate.i = 10,
                          inf.prob.q = 0.02,
                          act.rate.q = 2.5,
                          prog.rate = 1/10,
                          quar.rate = 1/30,
                          hosp.rate = 1/100,
                          disch.rate = 1/15,
                          rec.rate = 0.071,  
                          arec.rate = 0.05,
                          prog.dist.scale = 5,
                          prog.dist.shape = 1.5,
                          quar.dist.scale = 1,
                          quar.dist.shape = 1,
                          hosp.dist.scale = 1,
                          hosp.dist.shape = 1,
                          disch.dist.scale = 1,
                          disch.dist.shape = 1,
                          rec.dist.scale = 35,
                          rec.dist.shape = 1.5,
                          arec.dist.scale = 35,
                          arec.dist.shape = 1.5,
                          fat.rate.base = 1/50,
                          hosp.cap = 40,
                          fat.rate.overcap = 1/25,
                          fat.tcoeff = 0.5,
                          vital = TRUE,
                          flare.inf.point = NULL, 
                          flare.inf.num = NULL, 
                          a.rate = (10.5/365)/1000,
                          a.prop.e = 0.01,
                          a.prop.i = 0.001,
                          a.prop.q = 0.01,
                          ds.rate = (7/365)/1000,
                          de.rate = (7/365)/1000,
                          di.rate = (7/365)/1000,
                          dq.rate = (7/365)/1000,
                          dh.rate = (20/365)/1000,
                          dr.rate = (7/365)/1000) {
    
    # Get arguments
    p <- list()
    passed <- names(as.list(match.call())[-1])
    p$usr.specified <- passed
    
    formal.args <- formals(sys.function())
    for (arg in names(formal.args)) {
        if (as.logical(mget(arg) != "")) {
            p[arg] <- list(get(arg))
        }
    }
    
    ## Defaults and checks
    if (is.null(p$act.rate)) {
        p$act.rate <- 1
    }
    p$vital <- ifelse(!is.null(p$a.rate) | !is.null(p$ds.rate) |
                          !is.null(p$di.rate) | !is.null(p$dr.rate), TRUE, FALSE)
    
    p$groups <- 1
    
    if (!is.null(p$inter.eff) && is.null(p$inter.start)) {
        p$inter.start <- 1
    }
    
    ## Output
    class(p) <- c("param.seiqhrf", "list")
    return(p)
}


