#' Generating time-varying parameter values
#'
#' Generate a sequence of parameter values of length \code{nstep}.
#' The user specify a number of "change points" in the sequence
#' the function fills out the rest of the sequence s.t. 
#' the shape between each pair of consecutive change points is linear. 
#' All values before the first element and after the last element in \code{timing} 
#' are equal to the first element last element in \code{vals} respectively. 
#'
#' @param nstep The number of time steps to generate parameter values for, interpreted as the time range of the simulation. 
#' @param vals A vector of change point values in the sequence.  
#' @param timing A vector of the indices specifying the location or step numbers at which the change points occur. 
#'               The length of \code{vals} and \code{timing} should be the same, and both smaller than \code{nsteps}.
#'
#' @return A sequence of length nsteps. 
#' 
#' @examples
#' round(vary_param(nstep = 10, vals = c(2.5, 3.4, 1.2), timing = c(2, 4, 7)), 2)
#'
#' @importFrom utils tail
#' @export
vary_param <- function(nstep, vals, timing) {

    stopifnot(length(vals) == length(timing))
    y <- list()

    for(t in seq(1:nstep)){
        if(t <= timing[1]){              # If before first jump set to val[1]
            y.t <- vals[1]
        }else if(t > utils::tail(timing, n=1)){ # If after last jump set to val[-1]
            y.t <- utils::tail(vals, n=1)
        }else{                           # If intermediate step, calculate...
            for(j in (2:length(timing))){
                if(t > timing[j - 1] & t <= timing[j]){
                    start <- vals[j - 1]
                    end <- vals[j]
                    start_t <- timing[j - 1]
                    end_t <- timing[j]
                    y.t <- start - (t-start_t)*(start - end)/(end_t - start_t)
                }
            }
        }
        y <- append(y, y.t)
    }

    return(unlist(y))

}


#' Merge icm
#'
#' Function to merge icms
#'
#' @param x First to merge
#' @param y Second to merge
#' @param ... other parameters
#'
#' @return Combined
#' 
#' @importFrom doParallel registerDoParallel
#' @keywords internal
merge.seiqhrf.icm <- function(x, y, ...) {
    
    ## Check structure
    if (length(x) != length(y) || !identical(names(x), names(y))) {
        stop("x and y have different structure")
    }
    if (x$control$nsims > 1 & y$control$nsims > 1 &
        !identical(sapply(x, class), sapply(y, class))) {
        stop("x and y have different structure")
    }
    
    ## Check params
    check1 <- identical(x$param, y$param)
    check2 <- identical(x$control[-which(names(x$control) %in% c("nsims", "currsim"))],
                        y$control[-which(names(y$control) %in% c("nsims", "currsim"))])
    
    if (check1 == FALSE) {
        stop("x and y have different parameters")
    }
    if (check2 == FALSE) {
        stop("x and y have different controls")
    }
    
    
    z <- x
    new.range <- (x$control$nsims + 1):(x$control$nsims + y$control$nsims)
    
    # Merge data
    for (i in 1:length(x$epi)) {
        if (x$control$nsims == 1) {
            x$epi[[i]] <- data.frame(x$epi[[i]])
        }
        if (y$control$nsims == 1) {
            y$epi[[i]] <- data.frame(y$epi[[i]])
        }
        z$epi[[i]] <- cbind(x$epi[[i]], y$epi[[i]])
        names(z$epi[[i]])[new.range] <- paste0("sim", new.range)
    }
    
    z$control$nsims <- max(new.range)
    
    return(z)
}

#' Get Epidemic Output from icm Model
#' get_prev.icm from EpiModel 1.8.0
#' @keywords internal
get_prev <- function(dat, at){
    if (at == 1) {
        dat$epi <- list()
        dat$epi$s.num <- sum(dat$attr$active == 1 & dat$attr$status == 
                                 "s" & dat$attr$group == 1)
        dat$epi$i.num <- sum(dat$attr$active == 1 & dat$attr$status == 
                                 "i" & dat$attr$group == 1)
        dat$epi$num <- dat$epi$s.num + dat$epi$i.num
    }
    else {
        dat$epi$s.num[at] <- sum(dat$attr$active == 1 & dat$attr$status == 
                                     "s" & dat$attr$group == 1)
        dat$epi$i.num[at] <- sum(dat$attr$active == 1 & dat$attr$status == 
                                     "i" & dat$attr$group == 1)
        dat$epi$num[at] <- dat$epi$s.num[at] + dat$epi$i.num[at]
    }
    return(dat)
}